% undistort images where calibration data is available
% converts to gray scale
%% Main program
%undistort images
% 
 close all 
 clear all 

%% camera information
cwd = (fileparts(mfilename('fullpath')))
% select proper calibration file
opts = {'Phantom 4 Video','Phantom 3 Pro Video','Phantom 4 Pro Video','Custom Calibration','No Calibration'}
qStr = 'Which camera was this image taken with?'
calfile = bttnChoiseDialog(opts,'Camera Model Selection', 1, qStr)
% Handle response
switch calfile
    case 1
        calfile = fullfile(cwd,'p4nonprocal.mat');
        load(calfile);
        cameraParams = calibrationSession;
        %focal length
        cfl = 3.61;
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 2
        calfile = fullfile(cwd,'p3procal.mat');
        load(calfile);
        cameraParams = calibrationSession;
        %focal length
        cfl = 3.61;
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 3
        calfile = fullfile(cwd,'p4procal.mat');
        load(calfile);
        cameraParams = calibrationSession;
        %focal length
        cfl = 8.80;
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 4
        [FileName,PathName] = uigetfile('*.mat ','Select custom calibration with *.mat extention; note: Name in workspace must be "calibrationSession"');
        calfile = fullfile(PathName,char(FileName)); %concatenate filename and pathname
        load(calfile); %%when loaded this file must be named 'calibrationSession' in the workspace
        cameraParams = calibrationSession;
        %focal length
        prompt = {'Camera Focal Length in mm (e.g. 3.61 for P3/P4 and 8.8 mm for P4 Pro):'};
        dlg_title = 'Camera Focal Length';
        num_lines = 1;
        defaultans = {'3.61'};
        answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
        cfl = str2double(answer{1});
        %pixel pitch
        pmm = cfl/cameraParams.FocalLength(1);
    case 5
        calfile = 'NA';
        %manual camera information entry
        %focal length
        prompt = {'Camera Focal Length in mm (e.g. 3.61 for P3/P4 and 8.8 mm for P4 Pro):'};
        dlg_title = 'Camera Focal Length';
        num_lines = 1;
        defaultans = {'3.61'};
        answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
        cfl = str2double(answer{1});
        %default camera specs are for a 4K DJI phantom 3 pro video camera:
        %http://www.sony.net/Products/SC-HP/cx_news_archives/img/pdf/vol_70/imx117cqt.pdf
        %camera pixel pitch
        prompt = {'Camera sensor pixel pitch in mm (e.g. .00155 for P3/P4 and .0024 for P4 Pro):'};
        dlg_title = 'Input';
        num_lines = 1;
        defaultans = {'0.00155'}; %phantom 3/4 is 1.55 micrometers
        answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
        pmm = str2double(answer{1});
        cameraParams = 'NA';
end 
          

%% whale image reading and measuring section
%select files
[fileName,pathName,numfiles] = multifileopen(); % call multifileopen to open files for manipulation
% numfiles = 1 %used for debugging purposes
whale_data = cell(numfiles(2),23);
wcnt = 1;
for kk = 1:numfiles(2) %iterate through all images
    fullFileName = fullfile(pathName,char(fileName(kk))); %concatenate filename and pathname
    img = imread(fullFileName); %read image
    %%%%% CHANGE GRAY CONVERSION HERE%%%%%%
    img = rgb2gray(img);
    %%%%% CHANGE GRAY CONVERSION HERE%%%%%%
    img2 = img;
    img = histeq(img);
    if cameraParams == 'NA' %if no calibration info skip
        img1 = img;
    else
        img1 = undistortImage(img, cameraParams); %undistorts images
    end 
    
    imgout = [fullFileName(1:end-4),'_undistort.jpg']
    imwrite(img1,imgout);  %# Save the frame data
    close all
    wcnt = wcnt+1; % global counter that is initialized at program start
    kk = kk+1;
end 

